# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .transactional import (
    TransactionalResource,
    AsyncTransactionalResource,
    TransactionalResourceWithRawResponse,
    AsyncTransactionalResourceWithRawResponse,
    TransactionalResourceWithStreamingResponse,
    AsyncTransactionalResourceWithStreamingResponse,
)

__all__ = ["SendResource", "AsyncSendResource"]


class SendResource(SyncAPIResource):
    @cached_property
    def transactional(self) -> TransactionalResource:
        return TransactionalResource(self._client)

    @cached_property
    def with_raw_response(self) -> SendResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return SendResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SendResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return SendResourceWithStreamingResponse(self)


class AsyncSendResource(AsyncAPIResource):
    @cached_property
    def transactional(self) -> AsyncTransactionalResource:
        return AsyncTransactionalResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSendResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSendResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSendResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return AsyncSendResourceWithStreamingResponse(self)


class SendResourceWithRawResponse:
    def __init__(self, send: SendResource) -> None:
        self._send = send

    @cached_property
    def transactional(self) -> TransactionalResourceWithRawResponse:
        return TransactionalResourceWithRawResponse(self._send.transactional)


class AsyncSendResourceWithRawResponse:
    def __init__(self, send: AsyncSendResource) -> None:
        self._send = send

    @cached_property
    def transactional(self) -> AsyncTransactionalResourceWithRawResponse:
        return AsyncTransactionalResourceWithRawResponse(self._send.transactional)


class SendResourceWithStreamingResponse:
    def __init__(self, send: SendResource) -> None:
        self._send = send

    @cached_property
    def transactional(self) -> TransactionalResourceWithStreamingResponse:
        return TransactionalResourceWithStreamingResponse(self._send.transactional)


class AsyncSendResourceWithStreamingResponse:
    def __init__(self, send: AsyncSendResource) -> None:
        self._send = send

    @cached_property
    def transactional(self) -> AsyncTransactionalResourceWithStreamingResponse:
        return AsyncTransactionalResourceWithStreamingResponse(self._send.transactional)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .contact_delete_params import ContactDeleteParams as ContactDeleteParams
from .contact_update_params import ContactUpdateParams as ContactUpdateParams
from .contact_delete_response import ContactDeleteResponse as ContactDeleteResponse
from .contact_update_response import ContactUpdateResponse as ContactUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ImportCreateParams", "Contact"]


class ImportCreateParams(TypedDict, total=False):
    contacts: Required[Iterable[Contact]]


class Contact(TypedDict, total=False):
    email: Required[str]
    """Contact email address (required)"""

    custom_fields: Annotated[Dict[str, object], PropertyInfo(alias="customFields")]
    """Custom fields as key-value pairs"""

    first_name: Annotated[str, PropertyInfo(alias="firstName")]
    """First name"""

    last_name: Annotated[str, PropertyInfo(alias="lastName")]
    """Last name"""

    subscribed: bool
    """Subscription status (default true)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.send import transactional_send_params
from ..._base_client import make_request_options
from ...types.send.transactional_send_response import TransactionalSendResponse
from ...types.send.transactional_documentation_response import TransactionalDocumentationResponse

__all__ = ["TransactionalResource", "AsyncTransactionalResource"]


class TransactionalResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TransactionalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return TransactionalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return TransactionalResourceWithStreamingResponse(self)

    def documentation(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionalDocumentationResponse:
        """Get API documentation and endpoint information"""
        return self._get(
            "/send/transactional",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionalDocumentationResponse,
        )

    def send(
        self,
        *,
        chat_id: str,
        to: Union[str, SequenceNotStr[str]],
        from_: str | Omit = omit,
        reply_to: str | Omit = omit,
        subject: str | Omit = omit,
        variables: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionalSendResponse:
        """
        Send a transactional email using a published email template.

        **Permission required:** `transactional` or `all`

        The template must be published (status: "live") in Brew before it can be used
        via API.

        Args:
          chat_id: The transactional email template ID (from Brew)

          to: Recipient email(s) - single email or array of up to 1000

          from_: Override default sender email

          reply_to: Reply-to email address

          subject: Override default subject line

          variables: Template variables as key-value pairs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/send/transactional",
            body=maybe_transform(
                {
                    "chat_id": chat_id,
                    "to": to,
                    "from_": from_,
                    "reply_to": reply_to,
                    "subject": subject,
                    "variables": variables,
                },
                transactional_send_params.TransactionalSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionalSendResponse,
        )


class AsyncTransactionalResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTransactionalResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionalResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionalResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return AsyncTransactionalResourceWithStreamingResponse(self)

    async def documentation(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionalDocumentationResponse:
        """Get API documentation and endpoint information"""
        return await self._get(
            "/send/transactional",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionalDocumentationResponse,
        )

    async def send(
        self,
        *,
        chat_id: str,
        to: Union[str, SequenceNotStr[str]],
        from_: str | Omit = omit,
        reply_to: str | Omit = omit,
        subject: str | Omit = omit,
        variables: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionalSendResponse:
        """
        Send a transactional email using a published email template.

        **Permission required:** `transactional` or `all`

        The template must be published (status: "live") in Brew before it can be used
        via API.

        Args:
          chat_id: The transactional email template ID (from Brew)

          to: Recipient email(s) - single email or array of up to 1000

          from_: Override default sender email

          reply_to: Reply-to email address

          subject: Override default subject line

          variables: Template variables as key-value pairs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/send/transactional",
            body=await async_maybe_transform(
                {
                    "chat_id": chat_id,
                    "to": to,
                    "from_": from_,
                    "reply_to": reply_to,
                    "subject": subject,
                    "variables": variables,
                },
                transactional_send_params.TransactionalSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionalSendResponse,
        )


class TransactionalResourceWithRawResponse:
    def __init__(self, transactional: TransactionalResource) -> None:
        self._transactional = transactional

        self.documentation = to_raw_response_wrapper(
            transactional.documentation,
        )
        self.send = to_raw_response_wrapper(
            transactional.send,
        )


class AsyncTransactionalResourceWithRawResponse:
    def __init__(self, transactional: AsyncTransactionalResource) -> None:
        self._transactional = transactional

        self.documentation = async_to_raw_response_wrapper(
            transactional.documentation,
        )
        self.send = async_to_raw_response_wrapper(
            transactional.send,
        )


class TransactionalResourceWithStreamingResponse:
    def __init__(self, transactional: TransactionalResource) -> None:
        self._transactional = transactional

        self.documentation = to_streamed_response_wrapper(
            transactional.documentation,
        )
        self.send = to_streamed_response_wrapper(
            transactional.send,
        )


class AsyncTransactionalResourceWithStreamingResponse:
    def __init__(self, transactional: AsyncTransactionalResource) -> None:
        self._transactional = transactional

        self.documentation = async_to_streamed_response_wrapper(
            transactional.documentation,
        )
        self.send = async_to_streamed_response_wrapper(
            transactional.send,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["ContactDeleteParams"]


class ContactDeleteParams(TypedDict, total=False):
    email: Required[str]
    """Email address of contact to delete"""

    delete_fields: Annotated[SequenceNotStr[str], PropertyInfo(alias="deleteFields")]
    """Specific custom fields to delete.

    If omitted, deletes entire contact. Cannot delete standard fields (firstName,
    lastName, subscribed, tags).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from brew_sdk import BrewSDK, AsyncBrewSDK
from tests.utils import assert_matches_type
from brew_sdk.types import ContactDeleteResponse, ContactUpdateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestContacts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: BrewSDK) -> None:
        contact = client.contacts.update(
            email="john@example.com",
        )
        assert_matches_type(ContactUpdateResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: BrewSDK) -> None:
        contact = client.contacts.update(
            email="john@example.com",
            custom_fields={
                "company": "bar",
                "department": "bar",
            },
            first_name="Johnny",
            last_name="lastName",
            subscribed=True,
        )
        assert_matches_type(ContactUpdateResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: BrewSDK) -> None:
        response = client.contacts.with_raw_response.update(
            email="john@example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contact = response.parse()
        assert_matches_type(ContactUpdateResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: BrewSDK) -> None:
        with client.contacts.with_streaming_response.update(
            email="john@example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contact = response.parse()
            assert_matches_type(ContactUpdateResponse, contact, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: BrewSDK) -> None:
        contact = client.contacts.delete(
            email="john@example.com",
        )
        assert_matches_type(ContactDeleteResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete_with_all_params(self, client: BrewSDK) -> None:
        contact = client.contacts.delete(
            email="john@example.com",
            delete_fields=["string"],
        )
        assert_matches_type(ContactDeleteResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: BrewSDK) -> None:
        response = client.contacts.with_raw_response.delete(
            email="john@example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contact = response.parse()
        assert_matches_type(ContactDeleteResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: BrewSDK) -> None:
        with client.contacts.with_streaming_response.delete(
            email="john@example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contact = response.parse()
            assert_matches_type(ContactDeleteResponse, contact, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncContacts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncBrewSDK) -> None:
        contact = await async_client.contacts.update(
            email="john@example.com",
        )
        assert_matches_type(ContactUpdateResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncBrewSDK) -> None:
        contact = await async_client.contacts.update(
            email="john@example.com",
            custom_fields={
                "company": "bar",
                "department": "bar",
            },
            first_name="Johnny",
            last_name="lastName",
            subscribed=True,
        )
        assert_matches_type(ContactUpdateResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncBrewSDK) -> None:
        response = await async_client.contacts.with_raw_response.update(
            email="john@example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contact = await response.parse()
        assert_matches_type(ContactUpdateResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncBrewSDK) -> None:
        async with async_client.contacts.with_streaming_response.update(
            email="john@example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contact = await response.parse()
            assert_matches_type(ContactUpdateResponse, contact, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncBrewSDK) -> None:
        contact = await async_client.contacts.delete(
            email="john@example.com",
        )
        assert_matches_type(ContactDeleteResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncBrewSDK) -> None:
        contact = await async_client.contacts.delete(
            email="john@example.com",
            delete_fields=["string"],
        )
        assert_matches_type(ContactDeleteResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncBrewSDK) -> None:
        response = await async_client.contacts.with_raw_response.delete(
            email="john@example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        contact = await response.parse()
        assert_matches_type(ContactDeleteResponse, contact, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncBrewSDK) -> None:
        async with async_client.contacts.with_streaming_response.delete(
            email="john@example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            contact = await response.parse()
            assert_matches_type(ContactDeleteResponse, contact, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .send import (
    SendResource,
    AsyncSendResource,
    SendResourceWithRawResponse,
    AsyncSendResourceWithRawResponse,
    SendResourceWithStreamingResponse,
    AsyncSendResourceWithStreamingResponse,
)
from .contacts import (
    ContactsResource,
    AsyncContactsResource,
    ContactsResourceWithRawResponse,
    AsyncContactsResourceWithRawResponse,
    ContactsResourceWithStreamingResponse,
    AsyncContactsResourceWithStreamingResponse,
)

__all__ = [
    "ContactsResource",
    "AsyncContactsResource",
    "ContactsResourceWithRawResponse",
    "AsyncContactsResourceWithRawResponse",
    "ContactsResourceWithStreamingResponse",
    "AsyncContactsResourceWithStreamingResponse",
    "SendResource",
    "AsyncSendResource",
    "SendResourceWithRawResponse",
    "AsyncSendResourceWithRawResponse",
    "SendResourceWithStreamingResponse",
    "AsyncSendResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ContactUpdateResponse", "Data", "Meta"]


class Data(BaseModel):
    created_at: Optional[int] = FieldInfo(alias="createdAt", default=None)
    """Unix timestamp in milliseconds"""

    custom_fields: Optional[Dict[str, object]] = FieldInfo(alias="customFields", default=None)

    email: Optional[str] = None

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)

    subscribed: Optional[bool] = None

    updated_at: Optional[int] = FieldInfo(alias="updatedAt", default=None)
    """Unix timestamp in milliseconds"""


class Meta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class ContactUpdateResponse(BaseModel):
    data: Data

    success: Literal[True]

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "ImportGetStatusResponse",
    "ImportStatusResponse",
    "ImportStatusResponseData",
    "ImportStatusResponseDataImport",
    "ImportStatusResponseMeta",
    "ImportListResponse",
    "ImportListResponseData",
    "ImportListResponseDataImport",
    "ImportListResponseMeta",
]


class ImportStatusResponseDataImport(BaseModel):
    id: Optional[str] = None

    added_count: Optional[int] = FieldInfo(alias="addedCount", default=None)

    created_at: Optional[int] = FieldInfo(alias="createdAt", default=None)

    error_count: Optional[int] = FieldInfo(alias="errorCount", default=None)

    error_message: Optional[str] = FieldInfo(alias="errorMessage", default=None)

    file_name: Optional[str] = FieldInfo(alias="fileName", default=None)

    processed_rows: Optional[int] = FieldInfo(alias="processedRows", default=None)

    status: Optional[Literal["pending", "processing", "completed", "failed"]] = None

    total_rows: Optional[int] = FieldInfo(alias="totalRows", default=None)

    updated_at: Optional[int] = FieldInfo(alias="updatedAt", default=None)

    updated_count: Optional[int] = FieldInfo(alias="updatedCount", default=None)


class ImportStatusResponseData(BaseModel):
    import_: Optional[ImportStatusResponseDataImport] = FieldInfo(alias="import", default=None)


class ImportStatusResponseMeta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class ImportStatusResponse(BaseModel):
    data: ImportStatusResponseData

    success: Literal[True]

    meta: Optional[ImportStatusResponseMeta] = None


class ImportListResponseDataImport(BaseModel):
    id: Optional[str] = None

    added_count: Optional[int] = FieldInfo(alias="addedCount", default=None)

    created_at: Optional[int] = FieldInfo(alias="createdAt", default=None)

    error_count: Optional[int] = FieldInfo(alias="errorCount", default=None)

    file_name: Optional[str] = FieldInfo(alias="fileName", default=None)

    processed_rows: Optional[int] = FieldInfo(alias="processedRows", default=None)

    status: Optional[str] = None

    total_rows: Optional[int] = FieldInfo(alias="totalRows", default=None)

    updated_at: Optional[int] = FieldInfo(alias="updatedAt", default=None)

    updated_count: Optional[int] = FieldInfo(alias="updatedCount", default=None)


class ImportListResponseData(BaseModel):
    imports: Optional[List[ImportListResponseDataImport]] = None


class ImportListResponseMeta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class ImportListResponse(BaseModel):
    data: ImportListResponseData

    success: Literal[True]

    meta: Optional[ImportListResponseMeta] = None


ImportGetStatusResponse: TypeAlias = Union[ImportStatusResponse, ImportListResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["TransactionalSendResponse", "Data", "DataDroppedRecipient", "DataResult", "Meta", "Warning"]


class DataDroppedRecipient(BaseModel):
    email: Optional[str] = None
    """Blocked email address"""

    reason: Optional[str] = None
    """Reason for block (e.g., "bounced", "unsubscribed", "suppressed")"""


class DataResult(BaseModel):
    id: Optional[str] = None
    """Email ID (if successful)"""

    email: Optional[str] = None

    error: Optional[str] = None
    """Error message (if failed)"""


class Data(BaseModel):
    dropped: Optional[int] = None
    """Number of recipients blocked from sending (suppressed, bounced, etc.)"""

    dropped_recipients: Optional[List[DataDroppedRecipient]] = FieldInfo(alias="droppedRecipients", default=None)
    """Recipients that were blocked from sending"""

    failed: Optional[int] = None
    """Number of emails that failed to send"""

    results: Optional[List[DataResult]] = None

    sent: Optional[int] = None
    """Number of emails sent successfully"""


class Meta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class Warning(BaseModel):
    """Present if some recipients were dropped"""

    code: Optional[Literal["RECIPIENTS_DROPPED"]] = None
    """Warning code"""

    message: Optional[str] = None
    """Human-readable warning message"""


class TransactionalSendResponse(BaseModel):
    data: Data

    success: bool
    """True if at least one email was sent successfully"""

    meta: Optional[Meta] = None

    warning: Optional[Warning] = None
    """Present if some recipients were dropped"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .import_ import (
    ImportResource,
    AsyncImportResource,
    ImportResourceWithRawResponse,
    AsyncImportResourceWithRawResponse,
    ImportResourceWithStreamingResponse,
    AsyncImportResourceWithStreamingResponse,
)
from .contacts import (
    ContactsResource,
    AsyncContactsResource,
    ContactsResourceWithRawResponse,
    AsyncContactsResourceWithRawResponse,
    ContactsResourceWithStreamingResponse,
    AsyncContactsResourceWithStreamingResponse,
)

__all__ = [
    "ImportResource",
    "AsyncImportResource",
    "ImportResourceWithRawResponse",
    "AsyncImportResourceWithRawResponse",
    "ImportResourceWithStreamingResponse",
    "AsyncImportResourceWithStreamingResponse",
    "ContactsResource",
    "AsyncContactsResource",
    "ContactsResourceWithRawResponse",
    "AsyncContactsResourceWithRawResponse",
    "ContactsResourceWithStreamingResponse",
    "AsyncContactsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["TransactionalSendParams"]


class TransactionalSendParams(TypedDict, total=False):
    chat_id: Required[Annotated[str, PropertyInfo(alias="chatId")]]
    """The transactional email template ID (from Brew)"""

    to: Required[Union[str, SequenceNotStr[str]]]
    """Recipient email(s) - single email or array of up to 1000"""

    from_: Annotated[str, PropertyInfo(alias="from")]
    """Override default sender email"""

    reply_to: Annotated[str, PropertyInfo(alias="replyTo")]
    """Reply-to email address"""

    subject: str
    """Override default subject line"""

    variables: Dict[str, object]
    """Template variables as key-value pairs"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ImportCreateResponse", "Data", "DataError", "DataStats", "Meta"]


class DataError(BaseModel):
    email: Optional[str] = None

    error: Optional[str] = None

    index: Optional[int] = None


class DataStats(BaseModel):
    invalid: Optional[int] = None
    """Invalid contacts (validation errors)"""

    total: Optional[int] = None
    """Total contacts in request"""

    valid: Optional[int] = None
    """Valid contacts queued for import"""


class Data(BaseModel):
    errors: Optional[List[DataError]] = None
    """First 10 validation errors"""

    import_id: Optional[str] = FieldInfo(alias="importId", default=None)
    """Import job ID for status tracking"""

    stats: Optional[DataStats] = None


class Meta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class ImportCreateResponse(BaseModel):
    data: Data

    success: Literal[True]

    message: Optional[str] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .send import (
    SendResource,
    AsyncSendResource,
    SendResourceWithRawResponse,
    AsyncSendResourceWithRawResponse,
    SendResourceWithStreamingResponse,
    AsyncSendResourceWithStreamingResponse,
)
from .transactional import (
    TransactionalResource,
    AsyncTransactionalResource,
    TransactionalResourceWithRawResponse,
    AsyncTransactionalResourceWithRawResponse,
    TransactionalResourceWithStreamingResponse,
    AsyncTransactionalResourceWithStreamingResponse,
)

__all__ = [
    "TransactionalResource",
    "AsyncTransactionalResource",
    "TransactionalResourceWithRawResponse",
    "AsyncTransactionalResourceWithRawResponse",
    "TransactionalResourceWithStreamingResponse",
    "AsyncTransactionalResourceWithStreamingResponse",
    "SendResource",
    "AsyncSendResource",
    "SendResourceWithRawResponse",
    "AsyncSendResourceWithRawResponse",
    "SendResourceWithStreamingResponse",
    "AsyncSendResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .import_create_params import ImportCreateParams as ImportCreateParams
from .import_create_response import ImportCreateResponse as ImportCreateResponse
from .import_get_status_params import ImportGetStatusParams as ImportGetStatusParams
from .import_get_status_response import ImportGetStatusResponse as ImportGetStatusResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Iterable, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.contacts import import_create_params, import_get_status_params
from ...types.contacts.import_create_response import ImportCreateResponse
from ...types.contacts.import_get_status_response import ImportGetStatusResponse

__all__ = ["ImportResource", "AsyncImportResource"]


class ImportResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ImportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ImportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ImportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return ImportResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        contacts: Iterable[import_create_params.Contact],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportCreateResponse:
        """Import multiple contacts via API.

        Contacts are queued for processing.

        **Permission required:** `contacts` or `all`

        **Note:** Automations are triggered via the dedicated endpoint
        (`/automations/{automationId}/trigger`), not through contact imports.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/contacts/import",
            body=maybe_transform({"contacts": contacts}, import_create_params.ImportCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImportCreateResponse,
        )

    def get_status(
        self,
        *,
        import_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportGetStatusResponse:
        """
        Get the status of an import job or list recent imports.

        **Permission required:** `contacts` or `all`

        Args:
          import_id: Import job ID (omit to list recent imports)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ImportGetStatusResponse,
            self._get(
                "/contacts/import",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform({"import_id": import_id}, import_get_status_params.ImportGetStatusParams),
                ),
                cast_to=cast(
                    Any, ImportGetStatusResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncImportResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncImportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncImportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncImportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return AsyncImportResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        contacts: Iterable[import_create_params.Contact],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportCreateResponse:
        """Import multiple contacts via API.

        Contacts are queued for processing.

        **Permission required:** `contacts` or `all`

        **Note:** Automations are triggered via the dedicated endpoint
        (`/automations/{automationId}/trigger`), not through contact imports.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/contacts/import",
            body=await async_maybe_transform({"contacts": contacts}, import_create_params.ImportCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ImportCreateResponse,
        )

    async def get_status(
        self,
        *,
        import_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ImportGetStatusResponse:
        """
        Get the status of an import job or list recent imports.

        **Permission required:** `contacts` or `all`

        Args:
          import_id: Import job ID (omit to list recent imports)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ImportGetStatusResponse,
            await self._get(
                "/contacts/import",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {"import_id": import_id}, import_get_status_params.ImportGetStatusParams
                    ),
                ),
                cast_to=cast(
                    Any, ImportGetStatusResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ImportResourceWithRawResponse:
    def __init__(self, import_: ImportResource) -> None:
        self._import_ = import_

        self.create = to_raw_response_wrapper(
            import_.create,
        )
        self.get_status = to_raw_response_wrapper(
            import_.get_status,
        )


class AsyncImportResourceWithRawResponse:
    def __init__(self, import_: AsyncImportResource) -> None:
        self._import_ = import_

        self.create = async_to_raw_response_wrapper(
            import_.create,
        )
        self.get_status = async_to_raw_response_wrapper(
            import_.get_status,
        )


class ImportResourceWithStreamingResponse:
    def __init__(self, import_: ImportResource) -> None:
        self._import_ = import_

        self.create = to_streamed_response_wrapper(
            import_.create,
        )
        self.get_status = to_streamed_response_wrapper(
            import_.get_status,
        )


class AsyncImportResourceWithStreamingResponse:
    def __init__(self, import_: AsyncImportResource) -> None:
        self._import_ = import_

        self.create = async_to_streamed_response_wrapper(
            import_.create,
        )
        self.get_status = async_to_streamed_response_wrapper(
            import_.get_status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["TransactionalDocumentationResponse", "Endpoint"]


class Endpoint(BaseModel):
    description: Optional[str] = None

    methods: Optional[List[str]] = None

    path: Optional[str] = None


class TransactionalDocumentationResponse(BaseModel):
    """API documentation and endpoint information"""

    documentation: Optional[str] = None
    """Link to full API documentation"""

    endpoints: Optional[List[Endpoint]] = None
    """List of available endpoints"""

    version: Optional[str] = None
    """API version"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Dict, cast

import httpx

from ...types import contact_delete_params, contact_update_params
from .import_ import (
    ImportResource,
    AsyncImportResource,
    ImportResourceWithRawResponse,
    AsyncImportResourceWithRawResponse,
    ImportResourceWithStreamingResponse,
    AsyncImportResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.contact_delete_response import ContactDeleteResponse
from ...types.contact_update_response import ContactUpdateResponse

__all__ = ["ContactsResource", "AsyncContactsResource"]


class ContactsResource(SyncAPIResource):
    @cached_property
    def import_(self) -> ImportResource:
        return ImportResource(self._client)

    @cached_property
    def with_raw_response(self) -> ContactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ContactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return ContactsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        email: str,
        custom_fields: Dict[str, object] | Omit = omit,
        first_name: str | Omit = omit,
        last_name: str | Omit = omit,
        subscribed: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContactUpdateResponse:
        """Update a contact by email address.

        Custom fields are merged with existing values
        (not replaced).

        **Permission required:** `contacts` or `all`

        **Protected fields (cannot be updated):**

        - `email` (used as lookup key)
        - `createdAt`
        - `updatedAt`

        **Note:** Automations are now triggered via the dedicated automation trigger
        endpoint (`/automations/{automationId}/trigger`), not through contact updates.

        Args:
          email: Email address of contact to update (lookup key)

          custom_fields: Custom fields to merge (not replace)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/contacts",
            body=maybe_transform(
                {
                    "email": email,
                    "custom_fields": custom_fields,
                    "first_name": first_name,
                    "last_name": last_name,
                    "subscribed": subscribed,
                },
                contact_update_params.ContactUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContactUpdateResponse,
        )

    def delete(
        self,
        *,
        email: str,
        delete_fields: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContactDeleteResponse:
        """
        Delete a contact by email, or delete specific custom fields from a contact.

        **Permission required:** `contacts` or `all`

        If `deleteFields` is provided, only those custom fields are removed. If
        `deleteFields` is omitted, the entire contact is deleted.

        Args:
          email: Email address of contact to delete

          delete_fields: Specific custom fields to delete. If omitted, deletes entire contact. Cannot
              delete standard fields (firstName, lastName, subscribed, tags).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ContactDeleteResponse,
            self._delete(
                "/contacts",
                body=maybe_transform(
                    {
                        "email": email,
                        "delete_fields": delete_fields,
                    },
                    contact_delete_params.ContactDeleteParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ContactDeleteResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncContactsResource(AsyncAPIResource):
    @cached_property
    def import_(self) -> AsyncImportResource:
        return AsyncImportResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncContactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncContactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/GetBrew/brew-python-sdk#with_streaming_response
        """
        return AsyncContactsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        email: str,
        custom_fields: Dict[str, object] | Omit = omit,
        first_name: str | Omit = omit,
        last_name: str | Omit = omit,
        subscribed: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContactUpdateResponse:
        """Update a contact by email address.

        Custom fields are merged with existing values
        (not replaced).

        **Permission required:** `contacts` or `all`

        **Protected fields (cannot be updated):**

        - `email` (used as lookup key)
        - `createdAt`
        - `updatedAt`

        **Note:** Automations are now triggered via the dedicated automation trigger
        endpoint (`/automations/{automationId}/trigger`), not through contact updates.

        Args:
          email: Email address of contact to update (lookup key)

          custom_fields: Custom fields to merge (not replace)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/contacts",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "custom_fields": custom_fields,
                    "first_name": first_name,
                    "last_name": last_name,
                    "subscribed": subscribed,
                },
                contact_update_params.ContactUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContactUpdateResponse,
        )

    async def delete(
        self,
        *,
        email: str,
        delete_fields: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContactDeleteResponse:
        """
        Delete a contact by email, or delete specific custom fields from a contact.

        **Permission required:** `contacts` or `all`

        If `deleteFields` is provided, only those custom fields are removed. If
        `deleteFields` is omitted, the entire contact is deleted.

        Args:
          email: Email address of contact to delete

          delete_fields: Specific custom fields to delete. If omitted, deletes entire contact. Cannot
              delete standard fields (firstName, lastName, subscribed, tags).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return cast(
            ContactDeleteResponse,
            await self._delete(
                "/contacts",
                body=await async_maybe_transform(
                    {
                        "email": email,
                        "delete_fields": delete_fields,
                    },
                    contact_delete_params.ContactDeleteParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ContactDeleteResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ContactsResourceWithRawResponse:
    def __init__(self, contacts: ContactsResource) -> None:
        self._contacts = contacts

        self.update = to_raw_response_wrapper(
            contacts.update,
        )
        self.delete = to_raw_response_wrapper(
            contacts.delete,
        )

    @cached_property
    def import_(self) -> ImportResourceWithRawResponse:
        return ImportResourceWithRawResponse(self._contacts.import_)


class AsyncContactsResourceWithRawResponse:
    def __init__(self, contacts: AsyncContactsResource) -> None:
        self._contacts = contacts

        self.update = async_to_raw_response_wrapper(
            contacts.update,
        )
        self.delete = async_to_raw_response_wrapper(
            contacts.delete,
        )

    @cached_property
    def import_(self) -> AsyncImportResourceWithRawResponse:
        return AsyncImportResourceWithRawResponse(self._contacts.import_)


class ContactsResourceWithStreamingResponse:
    def __init__(self, contacts: ContactsResource) -> None:
        self._contacts = contacts

        self.update = to_streamed_response_wrapper(
            contacts.update,
        )
        self.delete = to_streamed_response_wrapper(
            contacts.delete,
        )

    @cached_property
    def import_(self) -> ImportResourceWithStreamingResponse:
        return ImportResourceWithStreamingResponse(self._contacts.import_)


class AsyncContactsResourceWithStreamingResponse:
    def __init__(self, contacts: AsyncContactsResource) -> None:
        self._contacts = contacts

        self.update = async_to_streamed_response_wrapper(
            contacts.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            contacts.delete,
        )

    @cached_property
    def import_(self) -> AsyncImportResourceWithStreamingResponse:
        return AsyncImportResourceWithStreamingResponse(self._contacts.import_)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = [
    "ContactDeleteResponse",
    "DeleteContactResponse",
    "DeleteContactResponseData",
    "DeleteContactResponseMeta",
    "DeleteFieldsResponse",
    "DeleteFieldsResponseData",
    "DeleteFieldsResponseMeta",
]


class DeleteContactResponseData(BaseModel):
    deleted: Optional[Literal[True]] = None

    email: Optional[str] = None


class DeleteContactResponseMeta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class DeleteContactResponse(BaseModel):
    data: DeleteContactResponseData

    success: Literal[True]

    meta: Optional[DeleteContactResponseMeta] = None


class DeleteFieldsResponseData(BaseModel):
    deleted_fields: Optional[List[str]] = FieldInfo(alias="deletedFields", default=None)

    missing_fields: Optional[List[str]] = FieldInfo(alias="missingFields", default=None)
    """Fields that were requested but didn't exist"""


class DeleteFieldsResponseMeta(BaseModel):
    request_id: Optional[str] = FieldInfo(alias="requestId", default=None)


class DeleteFieldsResponse(BaseModel):
    data: DeleteFieldsResponseData

    success: Literal[True]

    meta: Optional[DeleteFieldsResponseMeta] = None


ContactDeleteResponse: TypeAlias = Union[DeleteContactResponse, DeleteFieldsResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from brew_sdk import BrewSDK, AsyncBrewSDK
from tests.utils import assert_matches_type
from brew_sdk.types.contacts import (
    ImportCreateResponse,
    ImportGetStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestImport:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: BrewSDK) -> None:
        import_ = client.contacts.import_.create(
            contacts=[{"email": "john@example.com"}, {"email": "jane@example.com"}],
        )
        assert_matches_type(ImportCreateResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: BrewSDK) -> None:
        response = client.contacts.import_.with_raw_response.create(
            contacts=[{"email": "john@example.com"}, {"email": "jane@example.com"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        import_ = response.parse()
        assert_matches_type(ImportCreateResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: BrewSDK) -> None:
        with client.contacts.import_.with_streaming_response.create(
            contacts=[{"email": "john@example.com"}, {"email": "jane@example.com"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            import_ = response.parse()
            assert_matches_type(ImportCreateResponse, import_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_status(self, client: BrewSDK) -> None:
        import_ = client.contacts.import_.get_status()
        assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_status_with_all_params(self, client: BrewSDK) -> None:
        import_ = client.contacts.import_.get_status(
            import_id="importId",
        )
        assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_status(self, client: BrewSDK) -> None:
        response = client.contacts.import_.with_raw_response.get_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        import_ = response.parse()
        assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_status(self, client: BrewSDK) -> None:
        with client.contacts.import_.with_streaming_response.get_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            import_ = response.parse()
            assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncImport:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncBrewSDK) -> None:
        import_ = await async_client.contacts.import_.create(
            contacts=[{"email": "john@example.com"}, {"email": "jane@example.com"}],
        )
        assert_matches_type(ImportCreateResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncBrewSDK) -> None:
        response = await async_client.contacts.import_.with_raw_response.create(
            contacts=[{"email": "john@example.com"}, {"email": "jane@example.com"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        import_ = await response.parse()
        assert_matches_type(ImportCreateResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncBrewSDK) -> None:
        async with async_client.contacts.import_.with_streaming_response.create(
            contacts=[{"email": "john@example.com"}, {"email": "jane@example.com"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            import_ = await response.parse()
            assert_matches_type(ImportCreateResponse, import_, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_status(self, async_client: AsyncBrewSDK) -> None:
        import_ = await async_client.contacts.import_.get_status()
        assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_status_with_all_params(self, async_client: AsyncBrewSDK) -> None:
        import_ = await async_client.contacts.import_.get_status(
            import_id="importId",
        )
        assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_status(self, async_client: AsyncBrewSDK) -> None:
        response = await async_client.contacts.import_.with_raw_response.get_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        import_ = await response.parse()
        assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_status(self, async_client: AsyncBrewSDK) -> None:
        async with async_client.contacts.import_.with_streaming_response.get_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            import_ = await response.parse()
            assert_matches_type(ImportGetStatusResponse, import_, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ContactUpdateParams"]


class ContactUpdateParams(TypedDict, total=False):
    email: Required[str]
    """Email address of contact to update (lookup key)"""

    custom_fields: Annotated[Dict[str, object], PropertyInfo(alias="customFields")]
    """Custom fields to merge (not replace)"""

    first_name: Annotated[str, PropertyInfo(alias="firstName")]

    last_name: Annotated[str, PropertyInfo(alias="lastName")]

    subscribed: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .transactional_send_params import TransactionalSendParams as TransactionalSendParams
from .transactional_send_response import TransactionalSendResponse as TransactionalSendResponse
from .transactional_documentation_response import (
    TransactionalDocumentationResponse as TransactionalDocumentationResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ImportGetStatusParams"]


class ImportGetStatusParams(TypedDict, total=False):
    import_id: Annotated[str, PropertyInfo(alias="importId")]
    """Import job ID (omit to list recent imports)"""
